@@ -22,6 +22,12 @@ from string import Template
 
 logging.basicConfig(level=logging.DEBUG)
 
+# Optional Imports
+try:
+    import ast
+except:
+    logging.debug("Python version <2.6. AST coult not be imported. ")
+
 try:
     import saga
 except:
@@ -91,19 +97,15 @@ class bigjob_agent:
         # linked under mpirun_rsh
         if default_dict.has_key("mpirun"):
             self.MPIRUN=default_dict["mpirun"]
+        
+        if default_dict.has_key("number_executor_threads"):
+            THREAD_POOL_SIZE=int(default_dict["number_executor_threads"])
+            
         self.OUTPUT_TAR=False
         if default_dict.has_key("create_output_tar"):
             self.OUTPUT_TAR=eval(default_dict["create_output_tar"])
             logger.debug("Create output tar: %r", self.OUTPUT_TAR)
         
-        self.LAUNCH_METHOD="ssh"                    
-        if default_dict.has_key("launch_method"):
-            self.LAUNCH_METHOD=self.__get_launch_method(default_dict["launch_method"])
-        
-        logging.debug("Launch Method: " + self.LAUNCH_METHOD + " mpi: " + self.MPIRUN + " shell: " + self.SHELL)
-        
-        # init rms (SGE/PBS)
-        self.init_rms()
         self.failed_polls = 0
         
         ##############################################################################
@@ -144,10 +146,10 @@ class bigjob_agent:
                 traceback.print_tb(exc_traceback, file=sys.stderr)
         elif (self.coordination_url.startswith("redis://")):
             try:
-                from coordination.bigjob_coordination_redis import bigjob_coordination      
-                logger.debug("Utilizing Redis Backend: " + self.coordination_url + ". Please make sure Redis server is configured in bigjob_coordination_redis.py")
-            except:
-                logger.error("Error loading pyredis.")
+                from coordination.bigjob_coordination_redis import bigjob_coordination
+                logger.debug("Utilizing Redis Backend: " + self.coordination_url + ".")
+	    except:
+                logger.error("Error loading pyredis. Check configuration in bigjob_coordination_redis.py.")		
         elif (self.coordination_url.startswith("tcp://")):
             try:
                 from coordination.bigjob_coordination_zmq import bigjob_coordination
@@ -169,9 +171,32 @@ class bigjob_agent:
         logger.debug("set state to : " +  str(bigjob.state.Running))
         self.coordination.set_pilot_state(self.base_url, str(bigjob.state.Running), False)
         self.pilot_description = self.coordination.get_pilot_description(self.base_url)
+        try:
+            self.pilot_description = ast.literal_eval(self.pilot_description)
+        except:
+            logger.warn("Unable to parse pilot description")
+            self.pilot_description = None
+             
+
+        ############################################################################
+        # Detect launch method 
+        self.LAUNCH_METHOD="ssh"                    
+        if default_dict.has_key("launch_method"):
+            self.LAUNCH_METHOD=default_dict["launch_method"]
+            
+        self.LAUNCH_METHOD=self.__get_launch_method(self.LAUNCH_METHOD)
+        
+        logging.debug("Launch Method: " + self.LAUNCH_METHOD + " mpi: " + self.MPIRUN + " shell: " + self.SHELL)
+        
+        # init rms (SGE/PBS)
+        self.init_rms()
         
         ##############################################################################
         # start background thread for polling new jobs and monitoring current jobs
+        # check whether user requested a certain threadpool size
+        if self.pilot_description!=None and self.pilot_description.has_key("number_executor_threads"):
+            THREAD_POOL_SIZE=int(self.pilot_description["number_executor_threads"])
+        logger.debug("Creating executor thread pool of size: %d"%(THREAD_POOL_SIZE))
         self.resource_lock=threading.RLock()
         self.threadpool = ThreadPool(THREAD_POOL_SIZE)
         
@@ -229,7 +254,7 @@ class bigjob_agent:
             if(os.environ.get("GLIDEIN_CPUS")!=None):
                 num_cpus = int(os.environ.get("GLIDEIN_CPUS"))
         except:
-             pass
+            pass
         for i in range(0, num_cpus): 
             self.freenodes.append("localhost\n")
         return self.freenodes            
@@ -238,7 +263,11 @@ class bigjob_agent:
         """ initialize free nodes list with dummy (for fork jobs)"""  
         logger.debug("Init nodefile from /proc/cpuinfo")
         try:
-            num_cpus = self.get_num_cpus()
+            num_cpus=1
+            if self.pilot_description != None:
+                num_cpus = self.pilot_description["number_of_processes"]
+            else:
+                num_cpus = self.get_num_cpus()
             for i in range(0, num_cpus): 
                 self.freenodes.append("localhost\n")
         except IOError:
@@ -269,12 +298,19 @@ class bigjob_agent:
         """ initialize free nodes list from PBS environment """
         logger.debug("Init nodeslist from PBS NODEFILE")
         if self.LAUNCH_METHOD == "aprun":
-            # Workaround for Kraken
+            # Workaround for Kraken and Hector
             # PBS_NODEFILE does only contain front node
             # thus we create a dummy node file with the respective 
             # number of slots
             # aprun does not rely on the nodefile for job launching
-            number_nodes =  os.environ.get("PBS_NNODES")
+            
+            # get number of requested slots from pilot description
+            number_of_requested_processes = self.pilot_description["number_of_processes"]
+            if os.environ.has_key("PBS_NNODES"):
+                # use PBS assigned node count if available
+                number_nodes =  os.environ.get("PBS_NNODES")
+            else:
+                number_nodes =  number_of_requested_processes
             self.freenodes=[]
             for i in range(0, int(number_nodes)):
                 slot = "slot-%d\n"%i
@@ -629,19 +665,24 @@ class bigjob_agent:
         job_counter = 0               
         while self.is_stopped(self.base_url)==False:     
             if len(self.freenodes)==0:
-                time.sleep(3)
+                time.sleep(1)
                 continue
-            logger.debug("Dequeue sub-job from PilotCompute queue: " + self.base_url)       
-            job_url=self.coordination.dequeue_job(self.base_url)
+            if self.cds_queue_url!=None:
+                logger.debug("Dequeue sub-job from PilotCompute queue: "  + self.base_url 
+                             + " AND ComputeDataService queue: " + self.cds_queue_url)       
+                job_url=self.coordination.dequeue_job(self.base_url, self.cds_queue_url)
+            else:
+                logger.debug("Dequeue sub-job from PilotCompute queue: " + self.base_url)
+                job_url=self.coordination.dequeue_job(self.base_url)
             logger.debug("Dequed:%s"%str(job_url))
-            if job_url==None:
-                if self.cds_queue_url!=None:
-                    logger.debug("Dequeue sub-job from ComputeDataServicequeue: " + self.cds_queue_url)       
-                    job_url=self.coordination.dequeue_job(self.cds_queue_url)
-                    logger.debug("Dequed:%s"%str(job_url))
-                if job_url==None:
-                    time.sleep(3)
-                    continue
+#             if job_url==None:
+#                 if self.cds_queue_url!=None:
+#                     logger.debug("Dequeue sub-job from ComputeDataServicequeue: " + self.cds_queue_url)       
+#                     job_url=self.coordination.dequeue_job(self.cds_queue_url)
+#                     logger.debug("Dequed:%s"%str(job_url))
+#                 if job_url==None:
+#                     time.sleep(3)
+#                     continue
             if job_url=="STOP":
                 break
             
@@ -651,7 +692,8 @@ class bigjob_agent:
             
             request = WorkRequest(self.start_new_job_in_thread, [job_url])
             self.threadpool.putRequest(request)
-            #time.sleep(1)
+            if self.coordination.get_queue_length(self.cds_queue_url)==0 and self.coordination.get_queue_length(self.base_url)==0:
+                time.sleep(1)
             
         # wait for termination of Worker Threads
         # self.threadpool.wait()   
