@@ -136,6 +136,18 @@ class ComputeDataServiceDecentral(ComputeDataService):
         return self.pilot_job_service
     
     
+    def get_details(self):
+        """ returns a list with dicts that contains the details of the Pilot Compute, 
+                - job state
+                - description
+                - ...        
+        """
+        pilot_details=[]
+        for pcs in self.pilot_job_services:
+            for pc in pcs.list_pilots():
+                pilot_details.append(pc.get_details())
+        return pilot_details
+    
     ###########################################################################
     # Compute Data Service private methods
     def __submit_cu(self, compute_unit):
@@ -150,7 +162,8 @@ class ComputeDataServiceDecentral(ComputeDataService):
         self.pcs_coordination_namespace=self.pilot_job_services[0].coordination_queue
         logger.debug("Submit CU to big-job via external queue: %s"%self.pcs_coordination_namespace)
         sj.submit_job(self.pcs_coordination_namespace, compute_unit.subjob_description)
-        compute_unit._update_subjob(sj)
+
+        compute_unit._update_subjob(sj ,"UNKNOWN")
         return compute_unit
     
     
