@@ -9,6 +9,8 @@ import os
 import traceback
 import logging
 import uuid
+from pilot.api.api import PilotError
+from bigjob.bigjob_manager import BigJobError
 
 
 """ import bigjob classes """
@@ -55,7 +57,7 @@ class PilotComputeService(PilotComputeService):
             self.id = self.PJS_ID_PREFIX+str(uuid.uuid1())
             self.url = os.path.join(self.coordination_url, "pcs", self.id)
             self.coordination_queue = "PilotComputeServiceQueue-" + str(self.id)
-            logger.debug("Created Pilot Compute Service: %s"%self.url)
+            logger.debug("Created Pilot Compute Service: %s" % self.url)
         else:
             logger.error("Reconnect to PilotComputeService currently not supported.")
         
@@ -70,10 +72,14 @@ class PilotComputeService(PilotComputeService):
             Return value:
             A PilotCompute object
         """
-        bj_dict = self.__translate_pj_bj_description(pilot_compute_description)
-        bj = self.__start_bigjob(bj_dict)
-        pj = PilotCompute(self, bj, pilot_compute_description)
-        self.pilot_computes.append(pj)
+        pj = None
+        try:
+            bj_dict = self.__translate_pj_bj_description(pilot_compute_description)
+            bj = self.__start_bigjob(bj_dict)
+            pj = PilotCompute(self, bj, pilot_compute_description)
+            self.pilot_computes.append(pj)
+        except BigJobError as bj_error:
+            raise PilotError(bj_error.args[0])
         return pj
            
     
@@ -121,6 +127,12 @@ class PilotComputeService(PilotComputeService):
             resource_description["queue"] = pilot_compute_description["queue"] 
         else:
             resource_description["queue"] = None
+
+        if pilot_compute_description.has_key("spmd_variation"):
+            resource_description["spmd_variation"] = pilot_compute_description["spmd_variation"] 
+        else:
+            resource_description["spmd_variation"] = None
+            
             
         if pilot_compute_description.has_key("project"):
             resource_description["project"] = pilot_compute_description["project"] 
@@ -167,6 +179,7 @@ class PilotComputeService(PilotComputeService):
                            walltime = walltime,
                            processes_per_node = ppn,
                            filetransfers = bj_filetransfer,
+			               spmd_variation = bj_dict["spmd_variation"],
                            external_queue = self.coordination_queue,
                            pilot_compute_description = bj_dict["pilot_compute_description"]
                            )
@@ -214,6 +227,7 @@ class PilotCompute(PilotCompute):
             referenced by the pilot_url.  
 
         """        
+
         self.__subjobs = []
         self.__pilot_compute_service = None
         if pilot_url==None:
@@ -230,13 +244,13 @@ class PilotCompute(PilotCompute):
         # ComputeDataServiceDecentral
         if self.__pilot_compute_service!=None:
             self.coordination_queue = pilot_compute_service.coordination_queue
-            
-        
+
+
     def cancel(self):
         """ Terminates the pilot """
         self.__bigjob.cancel()    
     
-    
+
     def get_state(self):
         """ Returns the state of the pilot """
         return self.__bigjob.get_state()    
@@ -265,6 +279,7 @@ class PilotCompute(PilotCompute):
         for i in sj_list:
             cu_list.append(ComputeUnit(cu_url=i))
         return cu_list
+
     
     
     def get_url(self):
@@ -310,8 +325,7 @@ class PilotCompute(PilotCompute):
             are ignored.
         """
         cu = ComputeUnit(compute_unit_description)
-        return self._submit_cu(cu)
-    
+        return self._submit_cu(cu, self.get_local_working_directory())
     
     def __repr__(self):
         return str(self.__bigjob)
@@ -320,13 +334,14 @@ class PilotCompute(PilotCompute):
     ###########################################################################
     # Internal methods
         
-    def _submit_cu(self, compute_unit):
+    def _submit_cu(self, compute_unit, parent_bj_workdir):
         """ Submits compute unit to Bigjob """
         logger.debug("Submit CU to big-job")
         sj = subjob()
         sj.submit_job(self.__bigjob.pilot_url, compute_unit.subjob_description)
         self.__subjobs.append(sj)
-        compute_unit._update_subjob(sj)
+        compute_unit._update_subjob(sj, parent_bj_workdir)
+
         return compute_unit
         
 
@@ -349,6 +364,8 @@ class ComputeUnit(ComputeUnit):
 
     def __init__(self, compute_unit_description=None, compute_data_service=None, cu_url=None):
         
+        self._local_working_directory = None
+
         if cu_url==None:
             self.id = self.CU_ID_PREFIX + str(uuid.uuid1())
             if compute_data_service!=None:
@@ -359,13 +376,17 @@ class ComputeUnit(ComputeUnit):
             self.compute_unit_description = compute_unit_description # CU Description
             self.subjob_description = self.__translate_cu_sj_description(compute_unit_description)
         else:
-            self.__subjob = subjob(subjob_url=cu_url)
-           
+            self.__subjob = subjob(subjob_url=cu_url)           
     
     def get_id(self):
         """Returns unique identifier of Compute Unit. Deprecated: Please use get_url() instead."""
         return self.id
     
+    def get_local_working_directory(self):
+        """ Returns the local working directory of this
+            PilotCompute object.
+        """
+        return self._local_working_directory
     
     def get_url(self):   
         """Returns URL of Compute Unit. This URL can be used to reconnect to this Compute Unit later on."""
@@ -388,6 +409,9 @@ class ComputeUnit(ComputeUnit):
         if self.__subjob != None:
             self.state = self.__subjob.get_state()
         return self.state
+
+    def get_subjob(self):
+        return self.__subjob
     
     
     def wait(self):
@@ -413,8 +437,12 @@ class ComputeUnit(ComputeUnit):
         self.compute_unit_description = compute_unit_description # CU Description
         self.subjob_description = self.__translate_cu_sj_description(compute_unit_description)
 
-    def _update_subjob(self, subjob):
+    def _update_subjob(self, subjob, parent_bj_workdir):
         self.__subjob = subjob
+
+        # at this point, we can also set the working directory for this CU
+        self._local_working_directory = "%s/%s" \
+            % (parent_bj_workdir, subjob.uuid)
         
     # INTERNAL
     def __translate_cu_sj_description(self, compute_unit_description):
@@ -427,8 +455,18 @@ class ComputeUnit(ComputeUnit):
             jd.spmd_variation = "single"
         if compute_unit_description.has_key("arguments"): 
             jd.arguments = compute_unit_description["arguments"]
+
         if compute_unit_description.has_key("environment"):
-            jd.environment = compute_unit_description["environment"] 
+
+            env = compute_unit_description["environment"]
+            if type(env) == dict:
+                # convet to 'old-style' argument list
+                env_list = list()
+                for (key, val) in env.iteritems():
+                    env_list.append("%s=%s" % (key, val))
+                jd.environment = env_list
+            else:
+                jd.environment = env
         
         # handling number of processes
         if compute_unit_description.has_key("number_of_processes"):
@@ -451,5 +489,3 @@ class ComputeUnit(ComputeUnit):
         if compute_unit_description.has_key("output_data"):
             jd.output_data=compute_unit_description["output_data"]            
         return jd
-        
-        
